"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const epicCurrency_1 = require("./epicCurrency");
class epicCurrencyList {
    // Constructor
    constructor(currencyList) {
        // Defaults
        this.standard = new epicCurrency_1.epicCurrency("USD", "United States Dollar", 1, "$");
        this.currencyList = {};
        this.changeStandard = (currencyCode) => {
            if (this.standard.val().code.toLowerCase() == currencyCode.toLowerCase())
                return this;
            let currency;
            if (currency = this.currencyList[currencyCode.toLowerCase()]) {
                let standard = this.standard;
                this.standard = currency;
                this.addCurrency(standard);
                this.removeCurrency(currencyCode.toLowerCase());
                for (let c in this.currencyList)
                    if (this.currencyList.hasOwnProperty(c))
                        this.currencyList[c].changeRate(this.currencyList[c].val().rate / currency.val().rate);
                this.standard.changeRate(1);
            }
            else
                throw new Error("Invalid Currency Code. Currency Not Found!");
            return this;
        };
        this.addCurrency = (currency) => {
            if (currency instanceof epicCurrency_1.epicCurrency) {
                if (this.standard.val().code.toLowerCase() !== currency.val().code.toLowerCase())
                    this.currencyList[currency.val().code.toLowerCase()] = currency;
                else
                    this.standard = currency.changeRate(1);
            }
            else if (typeof currency === "object" &&
                currency.hasOwnProperty("code") &&
                currency.hasOwnProperty("name") &&
                currency.hasOwnProperty("rate")) {
                this.addCurrency(new epicCurrency_1.epicCurrency(currency.code, currency.name, currency.rate, currency.symbol));
            }
            else
                throw new Error("Invalid Currency Object Has Been Received!");
            return this;
        };
        this.addCurrencies = (currencies) => {
            if (currencies instanceof Array)
                currencies.forEach((v) => this.addCurrency(v));
            else if (typeof currencies === "object")
                for (let i in currencies) {
                    if (currencies.hasOwnProperty(i))
                        this.addCurrency(currencies[i]);
                }
            else
                throw new Error("Invalid Currencies Provided!");
            return this;
        };
        this.removeCurrency = (currencyCode) => {
            delete this.currencyList[currencyCode.toLowerCase()];
            return this;
        };
        this.removeCurrencies = (currencyCodes) => {
            if (currencyCodes instanceof Array)
                currencyCodes.forEach((v) => this.removeCurrency(v));
            else
                throw new Error("Invalid Currency Codes Array Provided!");
            return this;
        };
        this.getList = (includeStandard = true, clone = false) => {
            let currencyList = this.currencyList;
            if (includeStandard)
                currencyList[this.standard.val().code.toLowerCase()] = this.standard;
            if (clone) {
                // Create A Cloned Currency List
                const newList = {};
                for (let i in currencyList)
                    if (currencyList.hasOwnProperty(i))
                        newList[i] = currencyList[i].clone();
                // Re-Assign Cloned Data
                currencyList = newList;
            }
            return currencyList;
        };
        this.getListJson = (includeStandard = true) => {
            const list = {};
            Object.values(this.getList(includeStandard)).forEach((currency) => {
                list[currency.val().code] = currency.val();
            });
            return list;
        };
        this.getStandard = () => {
            return this.standard;
        };
        this.get = (currencyCode) => {
            let currencies = this.getList();
            if (!currencies[currencyCode.toLowerCase()])
                throw new Error("Invalid Currency Code! Currency Not Found.");
            return currencies[currencyCode.toLowerCase()];
        };
        this.has = (currencyCode) => {
            if (this.getList()[currencyCode.toLowerCase()])
                return true;
            else
                return false;
        };
        this.clone = (cloneCurrencies = false) => {
            const standardTemp = this.standard.val().code;
            return (new epicCurrencyList(this.changeStandard("usd").getList(false, cloneCurrencies))).changeStandard(standardTemp);
        };
        if (currencyList)
            if (currencyList instanceof epicCurrencyList) {
                if (!currencyList.has(this.standard.val().code))
                    throw new Error(`Currency List Object Should Contain "${this.standard.val().code.toUpperCase()}" Currency!`);
                this.currencyList = Object.assign(this.currencyList, currencyList.changeStandard(this.standard.val().code).getList(false));
            }
            else if (currencyList instanceof Array) {
                this.addCurrencies(currencyList);
            }
            else if (typeof currencyList === "object") {
                this.addCurrencies(Object.values(currencyList));
            }
            else
                throw new Error("Invalid Currency List Type Provided!");
        return this;
    }
}
exports.epicCurrencyList = epicCurrencyList;
//# sourceMappingURL=epicCurrencyList.js.map