"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const numeral_1 = __importDefault(require("numeral"));
const epicCurrency_1 = require("./epicCurrency");
class epicMoney {
    // Constructor
    constructor(amount, currency) {
        // Defaults
        this.amount = 0;
        this.stringify = (format = "s a C", numberFormat = '0,0.00') => {
            let amount = numeral_1.default(this.amount).format(numberFormat);
            let currency = this.currency.val();
            format = format.replace('s', currency.symbol || '');
            format = format.replace('S', currency.symbol || '');
            format = format.replace('a', amount);
            format = format.replace('A', amount);
            format = format.replace('c', currency.code.toLowerCase());
            format = format.replace('C', currency.code.toUpperCase());
            return format;
        };
        this.to = (currency) => {
            if (this.currency !== currency) {
                this.amount = this.amount * (currency.val().rate / this.currency.val().rate);
                this.currency = currency;
            }
            return this;
        };
        this.val = () => {
            return this.amount;
        };
        this.getCurrency = () => {
            return this.currency;
        };
        this.divide = (amount) => {
            if (typeof amount == "number")
                this.amount = this.amount / amount;
            else if (amount instanceof epicMoney) {
                this.amount = this.amount / amount.clone().to(this.currency).val();
            }
            else
                throw new Error("Please Provide A Valid Number Or Money!");
            return this;
        };
        this.multiply = (amount) => {
            if (typeof amount == "number")
                this.amount = this.amount * amount;
            else if (amount instanceof epicMoney) {
                this.amount = this.amount * amount.clone().to(this.currency).val();
            }
            else
                throw new Error("Please Provide A Valid Number Or Money!");
            return this;
        };
        this.plus = (amount) => {
            if (typeof amount == "number")
                this.amount = this.amount + amount;
            else if (amount instanceof epicMoney) {
                this.amount = this.amount + amount.clone().to(this.currency).val();
            }
            else
                throw new Error("Please Provide A Valid Number Or Money!");
            return this;
        };
        this.minus = (amount) => {
            if (typeof amount == "number")
                this.amount = this.amount - amount;
            else if (amount instanceof epicMoney) {
                this.amount = this.amount - amount.clone().to(this.currency).val();
            }
            else
                throw new Error("Please Provide A Valid Number Or Money!");
            return this;
        };
        this.major = () => {
            return this.multiply(100);
        };
        this.minor = () => {
            return this.divide(100);
        };
        this.invert = () => {
            return this.multiply(-1);
        };
        this.double = (times = 1) => {
            return this.multiply(2 * times);
        };
        this.triple = (times = 1) => {
            return this.multiply(3 * times);
        };
        this.percent = (percentage) => {
            this.amount = this.amount / 100 * percentage;
            return this;
        };
        this.round = (precision = 1) => {
            precision = Math.pow(10, precision);
            this.amount = Math.round(this.amount * precision) / precision;
            return this;
        };
        this.roundUp = (precision = 1) => {
            precision = Math.pow(10, precision);
            this.amount = Math.ceil(this.amount * precision) / precision;
            return this;
        };
        this.roundDown = (precision = 1) => {
            precision = Math.pow(10, precision);
            this.amount = Math.floor(this.amount * precision) / precision;
            return this;
        };
        this.partition = (parts = 2) => {
            if (typeof parts !== "number")
                throw new Error(`Invalid Argument Value For 'parts' Has Been Provided! A Number Is Expected!`);
            // Validate Parts
            if (parts < 1)
                parts = 1;
            if (parts > 1) {
                const results = [];
                for (let i = 0; i < 2; i++) {
                    results.push(new epicMoney(this.clone().divide(parts).val(), this.currency));
                }
                return results;
            }
            else {
                return this;
            }
        };
        this.clone = () => {
            return new epicMoney(this.amount, this.currency);
        };
        if (typeof amount === "string" && isNaN(amount))
            throw new Error(`Invalid Amount Has Been Provided!`);
        else if (typeof amount === "string")
            amount = parseFloat(amount);
        if (typeof amount !== "number")
            throw new Error(`Amount Should Be A Valid Number! Value Of Type '${typeof amount}' Has Been Provieded.`);
        if (typeof currency === "undefined") {
            this.amount = amount;
            this.currency = new epicCurrency_1.epicCurrency("N/A", "Currency Not Available!", 1, "!");
        }
        else if (currency instanceof epicCurrency_1.epicCurrency) {
            this.amount = amount;
            this.currency = currency;
        }
        else
            throw new Error("Invalid Currency Has Been Provided!");
        return this;
    }
}
exports.epicMoney = epicMoney;
//# sourceMappingURL=epicMoney.js.map