"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const epicMoney_1 = require("./epicMoney");
const epicCurrency_1 = require("./epicCurrency");
const epicCurrencyList_1 = require("./epicCurrencyList");
class epicMoneyStack {
    // Constructor
    constructor(currencyList) {
        // Defaults
        this.currencyList = new epicCurrencyList_1.epicCurrencyList();
        this.moneyList = [];
        this.add = (money, options = { description: "N/A", quantity: 1 }) => {
            if (!(money instanceof epicMoney_1.epicMoney))
                throw new Error("Invalid Money Object Provided!");
            money = money.clone();
            this.moneyList.push({
                description: options.description,
                quantity: parseInt(options.quantity),
                type: "ADD",
                money: money,
                total: money.multiply(parseInt(options.quantity)),
                history: this.total(),
            });
            return this;
        };
        this.deduct = (money, options = { description: "N/A", quantity: 1 }) => {
            if (!(money instanceof epicMoney_1.epicMoney))
                throw new Error("Invalid Money Object Provided!");
            money = money.clone();
            this.moneyList.push({
                description: options.description,
                quantity: parseInt(options.quantity),
                type: "DEDUCT",
                money: money.invert(),
                total: money.multiply(parseInt(options.quantity)),
                history: this.total(),
            });
            return this;
        };
        this.stacks = () => {
            return this.moneyList;
        };
        this.import = (source) => {
            source.forEach((row) => {
                this.add(new epicMoney_1.epicMoney(row.amount || 0, this.currencyList.get(row.currency || this.currencyList.getStandard().val().code)), {
                    description: row.description || "N/A",
                    quantity: parseInt((row.quantity || 1)),
                });
            });
            return this;
        };
        this.export = () => {
            const statement = [];
            this.stacks().forEach((stack) => {
                statement.push({
                    description: stack.description,
                    currency: stack.money.getCurrency().val().code,
                    quantity: stack.quantity,
                    amount: stack.money.val(),
                    total: stack.total.val(),
                });
            });
            return statement;
        };
        this.total = (currency) => {
            if (currency) {
                if (!(currency instanceof epicCurrency_1.epicCurrency))
                    throw new Error("Invalid Currency Object Provided!");
            }
            else
                currency = this.currencyList.getStandard();
            let total = new epicMoney_1.epicMoney(0, currency);
            this.moneyList.forEach((stack) => {
                total.plus(stack.total);
            });
            return total;
        };
        if (currencyList && currencyList instanceof epicCurrencyList_1.epicCurrencyList)
            this.currencyList = currencyList;
        return this;
    }
}
exports.epicMoneyStack = epicMoneyStack;
//# sourceMappingURL=epicMoneyStack.js.map